# Copyright 2023 Le Filament (https://le-filament.com)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html)

from odoo import fields, models, tools


class HallContractSaleFlowReport(models.Model):
    _name = "hall.contract.sale.flow.report"
    _description = "Sales and Flow Statistics"
    _auto = False
    _order = "day_date desc"

    # ------------------------------------------------------
    # Fields declaration
    # ------------------------------------------------------
    # ==== Sale fields ====
    day_date = fields.Date("Date statistique", required=True)

    nb_receipts = fields.Integer("Nombre de tickets")
    nb_seats = fields.Integer("Nombre de couverts")
    sales_excl_taxes = fields.Monetary("Total HT")
    sales_incl_taxes = fields.Monetary("Total TTC")
    currency_id = fields.Many2one(comodel_name="res.currency", readonly=True)

    # ==== Contract fields ====
    hall_id = fields.Many2one(
        comodel_name="hall.hall", string="Site", readonly=True, index=True
    )

    # ==== Flow fields ====
    visitor = fields.Integer(string="Visiteurs", readonly=True)
    passerby = fields.Integer(string="Passants", readonly=True)

    # ==== Other fields ====
    day_name = fields.Selection(
        selection=[
            ("1", "Lundi"),
            ("2", "Mardi"),
            ("3", "Mercredi"),
            ("4", "Jeudi"),
            ("5", "Vendredi"),
            ("6", "Samedi"),
            ("7", "Dimanche"),
        ],
        string="Jour de la semaine",
        readonly=True,
        index=True
    )
    amount_untaxed_five = fields.Monetary("TVA 5,5% HT", readonly=True)
    amount_total_five = fields.Monetary("TVA 5,5% TTC", readonly=True)
    amount_untaxed_ten = fields.Monetary("TVA 10% HT", readonly=True)
    amount_total_ten = fields.Monetary("TVA 10% TTC", readonly=True)
    amount_untaxed_twenty = fields.Monetary("TVA 20% HT", readonly=True)
    amount_total_twenty = fields.Monetary("TVA 20% TTC", readonly=True)

    _depends = {
        "hall.contract.sale": [
            "contract_id",
            "day_date",
            "nb_receipts",
            "sales_excl_taxes",
            "sales_incl_taxes",
        ],
        "hall.contract": [
            "hall_id",
        ],
        "hall.flow": [
            "hall_id",
            "date",
            "flow_type",
            "count",
        ],
    }

    # ------------------------------------------------------
    # Computed fields
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Subqueries
    # ------------------------------------------------------
    # ==== Sales ====
    def _select_sales(self):
        return """
            SELECT
                c.hall_id,
                s.day_date,
                SUM(s.nb_receipts) as nb_receipts,
                SUM(s.nb_seats) as nb_seats,
                SUM(s.sales_excl_taxes) as sales_excl_taxes,
                SUM(s.sales_incl_taxes) as sales_incl_taxes,
                s.currency_id,
                SUM(tva5.total_without_taxes) as amount_untaxed_five,
                SUM(tva5.total_with_taxes) as amount_total_five,
                SUM(tva10.total_without_taxes) as amount_untaxed_ten,
                SUM(tva10.total_with_taxes) as amount_total_ten,
                SUM(tva20.total_without_taxes) as amount_untaxed_twenty,
                SUM(tva20.total_with_taxes) as amount_total_twenty
        """

    def _from_sales(self):
        return """
            FROM
                hall_contract_sale s
            LEFT JOIN
                hall_contract c ON c.id = s.contract_id
            LEFT JOIN
                hall_contract_sale_tax tva5 ON tva5.sale_id = s.id and tva5.tax_rate = 0.055
            LEFT JOIN
                hall_contract_sale_tax tva10 ON tva10.sale_id = s.id and tva10.tax_rate = 0.1
            LEFT JOIN
                hall_contract_sale_tax tva20 ON tva20.sale_id = s.id and tva20.tax_rate = 0.2
        """

    def _clause_sales(self):
        return """
            WHERE
                s.is_test_datas is false
        """

    def _group_sales(self):
        return """
            GROUP BY
                c.hall_id,
                s.day_date,
                s.currency_id
        """

    def _query_sales(self):
        return """
            (%s %s %s %s) AS hall_sales
        """ % (
            self._select_sales(),
            self._from_sales(),
            self._clause_sales(),
            self._group_sales(),
        )

    # ==== Flows ====
    def _select_flow(self):
        return """
            SELECT
                f.hall_id,
                f.date,
                SUM(CASE WHEN f.flow_type = 'passerby' THEN f.count ELSE 0 END) as passerby,
                SUM(CASE WHEN f.flow_type = 'visitor' THEN f.count ELSE 0 END) as visitor
        """

    def _from_flow(self):
        return """
            FROM
                hall_flow f
        """

    def _group_flow(self):
        return """
            GROUP BY
                f.hall_id,
                f.date
        """

    def _query_flow(self):
        return """
            (%s %s %s) AS hall_flow
        """ % (
            self._select_flow(),
            self._from_flow(),
            self._group_flow(),
        )

    # ------------------------------------------------------
    # Global Query
    # ------------------------------------------------------
    def _select(self):
        return """
            SELECT
                ROW_NUMBER() OVER (ORDER BY hall_sales.day_date, hall_sales.hall_id) AS id,
                hall_sales.*,
                hall_flow.visitor,
                hall_flow.passerby,
                extract(isodow from hall_sales.day_date)::varchar as day_name
        """

    def _from(self):
        return """
            FROM
                %s
            LEFT JOIN
                %s
                ON hall_sales.hall_id = hall_flow.hall_id
                AND hall_sales.day_date = hall_flow.date
        """ % (
            self._query_sales(),
            self._query_flow(),
        )

    def init(self):
        tools.drop_view_if_exists(self._cr, self._table)
        self._cr.execute(
            "CREATE view %s as %s %s" % (self._table, self._select(), self._from())
        )
