# Copyright 2023 Le Filament (https://le-filament.com)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html)

from odoo import api, fields, models, tools


class HallContractSalePaymentReport(models.Model):
    _name = "hall.contract.sale.payment.report"
    _description = "Payment statistics"
    _auto = False
    _order = "day_date desc"

    # ------------------------------------------------------
    # Fields declaration
    # ------------------------------------------------------
    # ==== Payment fields =====
    payment_method = fields.Char("Moyen de paiement", readonly=True)
    amount_total = fields.Monetary("Montant", readonly=True)
    currency_id = fields.Many2one(comodel_name="res.currency", readonly=True)

    # ==== Archive fields =====
    day_date = fields.Date("Date statistique", required=True)
    contract_id = fields.Many2one(
        comodel_name="hall.contract", string="Contrat", readonly=True
    )

    # ==== Contract fields ====
    hall_id = fields.Many2one(comodel_name="hall.hall", string="Site", readonly=True)
    tag_ids = fields.Many2many(
        comodel_name="hall.contract.tag",
        string="Etiquette",
        compute="_compute_tags",
        search="_search_tags",
    )
    partner_id = fields.Many2one(
        comodel_name="res.partner", string="Client", readonly=True
    )
    commercial_name = fields.Char("Enseigne", readonly=True)
    type_job = fields.Selection(
        selection=[
            ("food", "Restauration"),
            ("hybrid", "Hybride"),
            ("food_scholl", "Food school"),
            ("pop_up", "Pop up"),
        ],
        string="Type",
        readonly=True,
    )

    # ==== Other fields ====
    day_name = fields.Selection(
        selection=[
            ("1", "Lundi"),
            ("2", "Mardi"),
            ("3", "Mercredi"),
            ("4", "Jeudi"),
            ("5", "Vendredi"),
            ("6", "Samedi"),
            ("7", "Dimanche"),
        ],
        string="Jour de la semaine",
        readonly=True,
    )

    _depends = {
        "hall.contract.sale.payment": [
            "amount_total",
            "payment_method",
        ],
        "hall.contract.sale.archive": [
            "contract_id",
            "day_date",
        ],
        "hall.contract": [
            "hall_id",
            "partner_id",
            "commercial_name",
            "tag_ids",
        ],
    }

    # ------------------------------------------------------
    # Computed fields
    # ------------------------------------------------------
    @api.depends("contract_id")
    def _compute_tags(self):
        for s in self:
            s.tag_ids = s.contract_id.tag_ids

    def _search_tags(self, operator, value):
        return [("contract_id.tag_ids", operator, value)]

    # ------------------------------------------------------
    # Global Query
    # ------------------------------------------------------
    def _select(self):
        return """
            SELECT
                p.id,
                p.amount_total,
                p.payment_method,
                p.currency_id,
                a.day_date,
                a.contract_id,
                c.hall_id,
                c.partner_id,
                c.commercial_name,
                c.type_job,
                extract(isodow from a.day_date)::varchar as day_name
        """

    def _from(self):
        return """
            FROM
                hall_contract_sale_payment p
            LEFT JOIN
                hall_contract_sale_archive a ON a.id = p.archive_id
            LEFT JOIN
                hall_contract c ON c.id = a.contract_id
        """

    def _clause(self):
        return """
            WHERE
                a.is_test_datas is false
        """

    def init(self):
        tools.drop_view_if_exists(self._cr, self._table)
        self._cr.execute(
            "CREATE view %s as %s %s %s"
            % (self._table, self._select(), self._from(), self._clause())
        )
